@@ -13,6 +13,8 @@ void Qpoll(	const struct pollfd *fds , int nfds , int timeout 	);
 void Qsocket(	 int type 	);
 void Qfcntl(	int fd , int cmd , long arg 	);
 void Qconnect(	int fd , const struct sockaddr *addr , int addrlen 	);
+void Qbind(	int fd , const struct sockaddr *addr , int addrlen 	);
+void Qlisten(	int fd , int backlog 	);
 void Qclose(	int fd 	);
 void Qsendto(	int fd , const void *msg , int msglen , const struct sockaddr *addr , int addrlen 	);
 void Qrecvfrom(	int fd , int buflen , int addrlen 	);
@@ -25,7 +27,7 @@ void Tfailed(const char *why);
 void Toutputerr(void);
 void Tnomem(void);
 void Tfsyscallr(const char *fmt, ...) PRINTFFORMAT(1,2);
-void Tensureoutputfile(void);
+void Tensurerecordfile(void);
 void Tmust(const char *call, const char *arg, int cond);
 void Tvbf(const char *fmt, ...) PRINTFFORMAT(1,2);
 void Tvbvf(const char *fmt, va_list al);
