@@ -1,11 +1,14 @@
 #include <assert.h>
 #include <string.h>
-#include <fcntl.h>
 #include <errno.h>
 #include <stdlib.h>
+#include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#include <sys/time.h>
+#include <unistd.h>
+#include <fcntl.h>
 #include "harness.h"
 static FILE *Tinputfile, *Treportfile;
 static vbuf vb2;
@@ -29,7 +32,7 @@ static void Pcheckinput(void) {
   if (ferror(Tinputfile)) Tfailed("read test log input file");
   if (feof(Tinputfile)) Psyntax("eof at syscall reply");
 }
-static void Tensureinputfile(void) {
+void Tensurerecordfile(void) {
   const char *fdstr;
   int fd;
   int chars;
@@ -101,9 +104,11 @@ static void Pfdset(fd_set *set, int max) {
   char *ep;
   if (vb2.buf[vb2.used++] != '[') Psyntax("fd set start not [");
   FD_ZERO(set);
+  if (vb2.buf[vb2.used] == ']') { vb2.used++; return; }
   for (;;) {
     r= strtoul(vb2.buf+vb2.used,&ep,10);
     if (r>=max) Psyntax("fd set member > max");
+    if (ep == (char*)vb2.buf+vb2.used) Psyntax("empty entry in fd set");
     FD_SET(r,set);
     vb2.used= ep - (char*)vb2.buf;
     c= vb2.buf[vb2.used++];
@@ -191,7 +196,7 @@ static int Pbytes(byte *buf, int maxlen) {
 void Q_vb(void) {
   int r;
   const char *nl;
-  Tensureinputfile();
+  Tensurerecordfile();
   if (!adns__vbuf_ensure(&vb2,vb.used+2)) Tnomem();
   r= fread(vb2.buf,1,vb.used+2,Tinputfile);
   if (feof(Tinputfile)) {
@@ -213,14 +218,10 @@ void Q_vb(void) {
   nl= memchr(vb.buf,'\n',vb.used);
   fprintf(Treportfile," %.*s\n", (int)(nl ? nl - (const char*)vb.buf : vb.used), vb.buf);
 }
-int Hselect(
-	int max , fd_set *rfds , fd_set *wfds , fd_set *efds , struct timeval *to 
-	) {
+int Hselect(	int max , fd_set *rfds , fd_set *wfds , fd_set *efds , struct timeval *to 	) {
  int r, amtread;
  char *ep;
- Qselect(
-	max , rfds , wfds , efds , to 
-	);
+ Qselect(	max , rfds , wfds , efds , to 	);
  if (!adns__vbuf_ensure(&vb2,1000)) Tnomem();
  fgets(vb2.buf,vb2.avail,Tinputfile); Pcheckinput();
  Tensurereportfile();
@@ -249,14 +250,10 @@ int Hselect(
  return r;
 }
 #ifdef HAVE_POLL
-int Hpoll(
-	struct pollfd *fds , int nfds , int timeout 
-	) {
+int Hpoll(	struct pollfd *fds , int nfds , int timeout 	) {
  int r, amtread;
  char *ep;
- Qpoll(
-	fds , nfds , timeout 
-	);
+ Qpoll(	fds , nfds , timeout 	);
  if (!adns__vbuf_ensure(&vb2,1000)) Tnomem();
  fgets(vb2.buf,vb2.avail,Tinputfile); Pcheckinput();
  Tensurereportfile();
@@ -283,16 +280,12 @@ int Hpoll(
  return r;
 }
 #endif
-int Hsocket(
-	int domain , int type , int protocol 
-	) {
+int Hsocket(	int domain , int type , int protocol 	) {
  int r, amtread;
  char *ep;
 	Tmust("socket","domain",domain==AF_INET); 
   Tmust("socket","type",type==SOCK_STREAM || type==SOCK_DGRAM); 
- Qsocket(
-	 type 
-	);
+ Qsocket(	 type 	);
  if (!adns__vbuf_ensure(&vb2,1000)) Tnomem();
  fgets(vb2.buf,vb2.avail,Tinputfile); Pcheckinput();
  Tensurereportfile();
@@ -317,9 +310,7 @@ int Hsocket(
  P_updatetime();
  return r;
 }
-int Hfcntl(
-	int fd , int cmd , ... 
-	) {
+int Hfcntl(	int fd , int cmd , ... 	) {
  int r, amtread;
 	va_list al; long arg; 
   Tmust("fcntl","cmd",cmd==F_SETFL || cmd==F_GETFL);
@@ -328,9 +319,7 @@ int Hfcntl(
   } else {
     arg= 0;
   } 
- Qfcntl(
-	fd , cmd , arg 
-	);
+ Qfcntl(	fd , cmd , arg 	);
  if (!adns__vbuf_ensure(&vb2,1000)) Tnomem();
  fgets(vb2.buf,vb2.avail,Tinputfile); Pcheckinput();
  Tensurereportfile();
@@ -369,13 +358,9 @@ int Hfcntl(
  P_updatetime();
  return r;
 }
-int Hconnect(
-	int fd , const struct sockaddr *addr , int addrlen 
-	) {
+int Hconnect(	int fd , const struct sockaddr *addr , int addrlen 	) {
  int r, amtread;
- Qconnect(
-	fd , addr , addrlen 
-	);
+ Qconnect(	fd , addr , addrlen 	);
  if (!adns__vbuf_ensure(&vb2,1000)) Tnomem();
  fgets(vb2.buf,vb2.avail,Tinputfile); Pcheckinput();
  Tensurereportfile();
@@ -400,13 +385,63 @@ int Hconnect(
  P_updatetime();
  return r;
 }
-int Hclose(
-	int fd 
-	) {
+int Hbind(	int fd , const struct sockaddr *addr , int addrlen 	) {
  int r, amtread;
- Qclose(
-	fd 
-	);
+ Qbind(	fd , addr , addrlen 	);
+ if (!adns__vbuf_ensure(&vb2,1000)) Tnomem();
+ fgets(vb2.buf,vb2.avail,Tinputfile); Pcheckinput();
+ Tensurereportfile();
+ fprintf(Treportfile,"%s",vb2.buf);
+ amtread= strlen(vb2.buf);
+ if (amtread<=0 || vb2.buf[--amtread]!='\n')
+  Psyntax("badly formed line");
+ vb2.buf[amtread]= 0;
+ if (memcmp(vb2.buf," bind=",6)) Psyntax("syscall reply mismatch");
+ if (vb2.buf[6] == 'E') {
+  int e;
+  e= Perrno(vb2.buf+6);
+  P_updatetime();
+  errno= e;
+  return -1;
+ }
+  if (memcmp(vb2.buf+6,"OK",2)) Psyntax("success/fail not E* or OK");
+  vb2.used= 6+2;
+  r= 0;
+ assert(vb2.used <= amtread);
+ if (vb2.used != amtread) Psyntax("junk at end of line");
+ P_updatetime();
+ return r;
+}
+int Hlisten(	int fd , int backlog 	) {
+ int r, amtread;
+ Qlisten(	fd , backlog 	);
+ if (!adns__vbuf_ensure(&vb2,1000)) Tnomem();
+ fgets(vb2.buf,vb2.avail,Tinputfile); Pcheckinput();
+ Tensurereportfile();
+ fprintf(Treportfile,"%s",vb2.buf);
+ amtread= strlen(vb2.buf);
+ if (amtread<=0 || vb2.buf[--amtread]!='\n')
+  Psyntax("badly formed line");
+ vb2.buf[amtread]= 0;
+ if (memcmp(vb2.buf," listen=",8)) Psyntax("syscall reply mismatch");
+ if (vb2.buf[8] == 'E') {
+  int e;
+  e= Perrno(vb2.buf+8);
+  P_updatetime();
+  errno= e;
+  return -1;
+ }
+  if (memcmp(vb2.buf+8,"OK",2)) Psyntax("success/fail not E* or OK");
+  vb2.used= 8+2;
+  r= 0;
+ assert(vb2.used <= amtread);
+ if (vb2.used != amtread) Psyntax("junk at end of line");
+ P_updatetime();
+ return r;
+}
+int Hclose(	int fd 	) {
+ int r, amtread;
+ Qclose(	fd 	);
  if (!adns__vbuf_ensure(&vb2,1000)) Tnomem();
  fgets(vb2.buf,vb2.avail,Tinputfile); Pcheckinput();
  Tensurereportfile();
@@ -431,15 +466,11 @@ int Hclose(
  P_updatetime();
  return r;
 }
-int Hsendto(
-	int fd , const void *msg , int msglen , unsigned int flags , const struct sockaddr *addr , int addrlen 
-	) {
+int Hsendto(	int fd , const void *msg , int msglen , unsigned int flags , const struct sockaddr *addr , int addrlen 	) {
  int r, amtread;
  char *ep;
 	Tmust("sendto","flags",flags==0); 
- Qsendto(
-	fd , msg , msglen , addr , addrlen 
-	);
+ Qsendto(	fd , msg , msglen , addr , addrlen 	);
  if (!adns__vbuf_ensure(&vb2,1000)) Tnomem();
  fgets(vb2.buf,vb2.avail,Tinputfile); Pcheckinput();
  Tensurereportfile();
@@ -464,15 +495,11 @@ int Hsendto(
  P_updatetime();
  return r;
 }
-int Hrecvfrom(
-	int fd , void *buf , int buflen , unsigned int flags , struct sockaddr *addr , int *addrlen 
-	) {
+int Hrecvfrom(	int fd , void *buf , int buflen , unsigned int flags , struct sockaddr *addr , int *addrlen 	) {
  int r, amtread;
 	Tmust("recvfrom","flags",flags==0); 
 	Tmust("recvfrom","*addrlen",*addrlen>=sizeof(struct sockaddr_in)); 
- Qrecvfrom(
-	fd , buflen , *addrlen 
-	);
+ Qrecvfrom(	fd , buflen , *addrlen 	);
  if (!adns__vbuf_ensure(&vb2,1000)) Tnomem();
  fgets(vb2.buf,vb2.avail,Tinputfile); Pcheckinput();
  Tensurereportfile();
@@ -499,13 +526,9 @@ int Hrecvfrom(
  P_updatetime();
  return r;
 }
-int Hread(
-	int fd , void *buf , size_t buflen 
-	) {
+int Hread(	int fd , void *buf , size_t buflen 	) {
  int r, amtread;
- Qread(
-	fd , buflen 
-	);
+ Qread(	fd , buflen 	);
  if (!adns__vbuf_ensure(&vb2,1000)) Tnomem();
  fgets(vb2.buf,vb2.avail,Tinputfile); Pcheckinput();
  Tensurereportfile();
@@ -531,14 +554,10 @@ int Hread(
  P_updatetime();
  return r;
 }
-int Hwrite(
-	int fd , const void *buf , size_t len 
-	) {
+int Hwrite(	int fd , const void *buf , size_t len 	) {
  int r, amtread;
  char *ep;
- Qwrite(
-	fd , buf , len 
-	);
+ Qwrite(	fd , buf , len 	);
  if (!adns__vbuf_ensure(&vb2,1000)) Tnomem();
  fgets(vb2.buf,vb2.avail,Tinputfile); Pcheckinput();
  Tensurereportfile();
